package com.sample;

import java.io.InputStreamReader;
import java.io.Reader;

import junit.framework.Assert;

import org.drools.RuleBase;
import org.drools.RuleBaseFactory;
import org.drools.StatefulSession;
import org.drools.WorkingMemory;
import org.drools.compiler.PackageBuilder;
import org.drools.event.ActivationCancelledEvent;
import org.drools.event.ActivationCreatedEvent;
import org.drools.event.AfterActivationFiredEvent;
import org.drools.event.AfterFunctionRemovedEvent;
import org.drools.event.AfterPackageAddedEvent;
import org.drools.event.AfterPackageRemovedEvent;
import org.drools.event.AfterRuleAddedEvent;
import org.drools.event.AfterRuleBaseLockedEvent;
import org.drools.event.AfterRuleBaseUnlockedEvent;
import org.drools.event.AfterRuleRemovedEvent;
import org.drools.event.AgendaEventListener;
import org.drools.event.AgendaGroupPoppedEvent;
import org.drools.event.AgendaGroupPushedEvent;
import org.drools.event.BeforeActivationFiredEvent;
import org.drools.event.BeforeFunctionRemovedEvent;
import org.drools.event.BeforePackageAddedEvent;
import org.drools.event.BeforePackageRemovedEvent;
import org.drools.event.BeforeRuleAddedEvent;
import org.drools.event.BeforeRuleBaseLockedEvent;
import org.drools.event.BeforeRuleBaseUnlockedEvent;
import org.drools.event.BeforeRuleRemovedEvent;
import org.drools.event.RuleBaseEventListener;
import org.drools.rule.Package;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import com.sample.pojo.TicketCaisse;

public class JUnitSimpleRule {

	private RuleBase ruleBaseSimple = null;

	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
	}

	@AfterClass
	public static void tearDownAfterClass() throws Exception {
	}

	@Before
	public void setUp() throws Exception {
		ruleBaseSimple = RuleBaseFactory.newRuleBase();
		ruleBaseSimple.addEventListener(new RuleBaseEventListener() {

			public void afterFunctionRemoved(AfterFunctionRemovedEvent event) {
				// TODO Auto-generated method stub

			}

			public void afterPackageAdded(AfterPackageAddedEvent event) {
				// TODO Auto-generated method stub

			}

			public void afterPackageRemoved(AfterPackageRemovedEvent event) {
			}

			public void afterRuleAdded(AfterRuleAddedEvent event) {
				System.out.println("Rule Added " + event.getRule().getName());

			}

			public void afterRuleBaseLocked(AfterRuleBaseLockedEvent event) {
				// TODO Auto-generated method stub

			}

			public void afterRuleBaseUnlocked(AfterRuleBaseUnlockedEvent event) {
				// TODO Auto-generated method stub

			}

			public void afterRuleRemoved(AfterRuleRemovedEvent event) {
				System.out.println("Rule removed " + event.getRule().getName());

			}

			public void beforeFunctionRemoved(BeforeFunctionRemovedEvent event) {
				// TODO Auto-generated method stub

			}

			public void beforePackageAdded(BeforePackageAddedEvent event) {
				// TODO Auto-generated method stub

			}

			public void beforePackageRemoved(BeforePackageRemovedEvent event) {
				// TODO Auto-generated method stub

			}

			public void beforeRuleAdded(BeforeRuleAddedEvent event) {
				// TODO Auto-generated method stub

			}

			public void beforeRuleBaseLocked(BeforeRuleBaseLockedEvent event) {
				// TODO Auto-generated method stub

			}

			public void beforeRuleBaseUnlocked(BeforeRuleBaseUnlockedEvent event) {
				// TODO Auto-generated method stub

			}

			public void beforeRuleRemoved(BeforeRuleRemovedEvent event) {
				// TODO Auto-generated method stub

			}

		});
		ruleBaseSimple = addReadRule(ruleBaseSimple, "/SimpleRule.drl");
	}

	@After
	public void tearDown() throws Exception {
		ruleBaseSimple = null;
	}

	@Test
	public void testSimpleRule() throws Exception {
		StatefulSession workingMemory = ruleBaseSimple.newStatefulSession();
		TicketCaisse t = new TicketCaisse();
		workingMemory.insert(t);
		workingMemory.fireAllRules();
		Assert.assertTrue(t.getTicketID().equals("a"));
		Assert.assertTrue(t.getReductionTicket("b") != null);
		Assert
				.assertTrue(t.getReductionTicket("b").getMontantReduction() == 10.0);
		Assert.assertTrue(t.getMontantReduction() == 10.0);
		workingMemory.dispose();

	}

	@Test
	public void testDeleteSimpleRule() throws Exception {
		ruleBaseSimple.removeRule("fr.sopragroup", "c");
		StatefulSession workingMemory = ruleBaseSimple.newStatefulSession();
		TicketCaisse t = new TicketCaisse();
		workingMemory.insert(t);
		workingMemory.fireAllRules();
		Assert.assertTrue(t.getTicketID().equals("a"));
		Assert.assertTrue(t.getReductionTicket("b") != null);
		Assert
				.assertTrue(t.getReductionTicket("b").getMontantReduction() == 10.0);
		workingMemory.dispose();

	}

	private static Package getPackage(String fileName) throws Exception {
		Reader source = new InputStreamReader(JUnitSimpleRule.class
				.getResourceAsStream(fileName));

		// optionally read in the DSL (if you are using it).
		// Reader dsl = new InputStreamReader(
		// DroolsTest.class.getResourceAsStream( "/mylang.dsl" ) );

		// Use package builder to build up a rule package.
		// An alternative lower level class called "DrlParser" can also be
		// used...

		PackageBuilder builder = new PackageBuilder();

		// this wil parse and compile in one step
		// NOTE: There are 2 methods here, the one argument one is for normal
		// DRL.
		builder.addPackageFromDrl(source);

		// Use the following instead of above if you are using a DSL:
		// builder.addPackageFromDrl( source, dsl );

		// get the compiled package (which is serializable)
		Package pkg = builder.getPackage();
		return pkg;

	}

	private static RuleBase addReadRule(RuleBase ruleBase, String fileName)
			throws Exception {
		// get the compiled package (which is serializable)
		Package pkg = getPackage(fileName);

		// add the package to a rulebase (deploy the rule package).
		ruleBase.addPackage(pkg);
		return ruleBase;
	}
}
